const lo = require('lodash')
const consts = require('../../consts')
const hlpPolicy = require('../../helpers_policy')

const TaskTypeSettingsValue = 'PluginTaskType'
const taskConverters = {
    AntiCryptor: require('./types/AntiCryptor.js'),
    AutoRuleGenerator: require('./types/AutoRuleGenerator.js'),
    CheckAppIntegrity: require('./types/CheckAppIntegrity.js'),
    CheckFilesIntegrity: require('./types/CheckFilesIntegrity.js'),
    DeviceControl: require('./types/DeviceControl.js'),
    DeviceRuleGenerator: require('./types/DeviceRuleGenerator.js'),
    FileIntegrityMonitor: require('./types/FileIntegrityMonitor.js'),
    Firewall: require('./types/Firewall.js'),
    // HostBlocker
    ICAPStorageProtection: require('./types/ICAPStorageProtection.js'),
    InstallKey: require('./types/InstallKey.js'),
    KSNService: require('./types/KSNService.js'),
    LogInspector: require('./types/LogInspector.js'),
    NetAppAntiCryptor: require('./types/NetAppAntiCryptor.js'),
    NetAppMonitor: require('./types/NetAppMonitor.js'),
    NetworkAttackBlocker: require('./types/NetworkAttackBlocker.js'),
    OnAccess: require('./types/OnAccess.js'),
    OnDemand: require('./types/OnDemand.js'),
    // PlcCheck
    // PlcInvestigate
    ProgramExecutionPrevention: require('./types/ProgramExecutionPrevention.js'),
    RetranslateUpdates: require('./types/RetranslateUpdates.js'),
    ScanQuarantine: require('./types/ScanQuarantine.js'),
    ScriptChecker: require('./types/ScriptChecker.js'),
    UpdateBases: require('./types/UpdateBases.js'),
    UpdateComponents: require('./types/UpdateComponents.js'),
    WebTrafficMonitor: require('./types/WebTrafficMonitor.js')
    // WiFiControl
}

function LocalTaskSettings () {
    let curHostId = '';
    let curTaskId = '';
    let groupId = 0;
    let currentTaskType = '';
    let activePolicyId = -1;
    let taskAdditionalParams = {};
    let policiesCache = {};
    let rbacCache = {};
    let GroupTask = false;
    let LockTask = false;

    async function getTaskDataCache(kscApi, taskId, hostId) {
        let result = await hlpPolicy.GetTaskData(kscApi, taskId, hostId);
        if ( result && result.TASK_ADDITIONAL_PARAMS !==undefined ) {
            taskAdditionalParams = lo.clone(result.TASK_ADDITIONAL_PARAMS)
            currentTaskType = result.TASK_NAME;
            GroupTask = (result.TASK_INFO_PARAMS!==undefined && result.TASK_INFO_PARAMS.PRTS_TASK_GROUPID!=undefined) ? true : false;
        }
        else{
            curTaskId = ''
            currentTaskType = ''
            taskAdditionalParams = {}
            GroupTask = false;
        }
    }

    async function UpdateRbacState(api, task_type) {
        try{
            const ReadOnly  = await api.HstAccessControl.IsTaskTypeReadonly({
                lGroupId: groupId,
                szwProduct: consts.wstrProduct,
                szwVersion: consts.wstrVersion,
                szwTaskTypeName: task_type
            });
            if(ReadOnly.result !== undefined){
                LockTask = ReadOnly.result;
            }
        }
        catch (error) {
            console.error(error)
        }
    }

    /*
     * Called via api.js in getFormModel
     */
    async function getTaskData(kscApi, taskId, hostId, taskType, form, sGroupId) {
        if(curHostId!==hostId || curTaskId!= taskId) {
            curHostId = hostId;
            curTaskId = taskId;
            groupId = parseInt(sGroupId);
            taskAdditionalParams = {};
            policiesCache = {};
        }

        await UpdateRbacState(kscApi, taskType)

        // request from Root task page form with TaskType
        if(taskType!=='policy'){
            await getTaskDataCache(kscApi, taskId, hostId);
            activePolicyId = await hlpPolicy.GetHostActivePolicyId(kscApi, hostId);
            if (taskConverters[currentTaskType]!==undefined) {
                if(activePolicyId===-1){
                    hlpPolicy.UpdatePoliciesCacheAsDefault(taskConverters[currentTaskType].defPolicySections, policiesCache);
                } else {
                    await hlpPolicy.UpdatePoliciesCache(kscApi, activePolicyId, taskConverters[currentTaskType].defPolicySections, policiesCache);
                }
                if(LockTask && taskConverters[currentTaskType].RbacToPolicyCache!==undefined){
                    policiesCache = taskConverters[currentTaskType].RbacToPolicyCache(policiesCache);
                }
            }
        }
        // get data from cache
        let computedData = {};
        let converter = taskConverters[currentTaskType]
        if(converter) {
            if(converter.toViewEx!==undefined){
                computedData = await converter.toViewEx(kscApi, taskAdditionalParams, policiesCache, form, GroupTask, LockTask);
            } else{
                computedData = converter.toView(taskAdditionalParams, policiesCache, form, GroupTask, LockTask);
            }
            computedData[TaskTypeSettingsValue] = currentTaskType;
        }
        return computedData;
    }

    function getTaskAdditionalParams(TaskType) {
        console.log('kess_windows:LocalTask:getTaskAdditionalParams ' + TaskType + ', Params:' + JSON.stringify(taskAdditionalParams))
        return taskAdditionalParams;
    }

    async function updateTaskData ({kscApi, modelData, formType, hostId}) {
        let converter = taskConverters[currentTaskType]
        if (converter) {
            if(converter.toRawEx!==undefined){
                await converter.toRawEx(kscApi, taskAdditionalParams, modelData, formType, hostId);
            }
            else{
                converter.toRaw(taskAdditionalParams, modelData, formType);
            }
        }
    }

    return {
        getTaskData,
        getTaskAdditionalParams,
        updateTaskData
    }
}

module.exports = LocalTaskSettings
