const lo = require('lodash');
const hlp = require('../../../helpers');
const hlpPolicy = require('../../../helpers_policy');
const hlpSch = require('./TaskScheduleHelper.js');

const defPolicySections = ['AntiCryptorPolicy','PredefinedTasksPolicy'];
const formCommonValues = ['EnforcementMode','UseHeuristicLevel','HeuristicLevel'];
const formScopeValues = ['Mode','SpecificFolders'];
const formExcludesValues = ['UseExcludeFolders','ExcludeFolders'];

function toView(TASK_ADDITIONAL_PARAMS, activePolicyCache, formType) {
    if(formType === 'Schedule' ) {
        let data = hlpSch.toView(TASK_ADDITIONAL_PARAMS.Schedule, activePolicyCache.PredefinedTasksPolicy.AntiCryptorPolicy);
        data.LocalTask = true;
        return data;
    }

    if(formType!=='Main')
        return {}
    let scope = hlp.localSettingsToView(TASK_ADDITIONAL_PARAMS.Settings,activePolicyCache.AntiCryptorPolicy,formScopeValues);
    scope.UiFolders = scope.SpecificFolders.KLPRSS_Val[0].map(function(node) { return {"Name":node}; })
    scope.LocalTask = true;
    let excl = hlp.localSettingsToView(TASK_ADDITIONAL_PARAMS.Settings,activePolicyCache.AntiCryptorPolicy,formExcludesValues);
    excl.UiFolders = excl.ExcludeFolders.KLPRSS_Val[0].map(function(node) { return {"Name":node}; })
    excl.LocalTask = true;
    let data = {
        Form_Network_AntiCryptorCommon: hlp.localSettingsToView(TASK_ADDITIONAL_PARAMS.Settings,activePolicyCache.AntiCryptorPolicy,formCommonValues),
        Form_Network_AntiCryptorScope:scope,
        Form_Network_AntiCryptorExcludes:excl
    }
    data.Form_Network_AntiCryptorCommon.LocalTask = true;
    data.Form_Network_AntiCryptorScope.LocalTask = true;
    data.Form_Network_AntiCryptorExcludes.LocalTask = true;
    return data
}

function toRaw(TASK_ADDITIONAL_PARAMS, changedData, formType) {
    if(formType === 'Schedule' ) {
        hlpSch.toRaw(changedData, TASK_ADDITIONAL_PARAMS.Schedule);
        return;
    }

    if(formType!=='Main')
        return;
    const AntiCryptorCommon = hlp.ItemToLocalSettings(changedData.Form_Network_AntiCryptorCommon);
    Object.assign(TASK_ADDITIONAL_PARAMS.Settings, lo.pick(AntiCryptorCommon, formCommonValues))

    const AntiCryptorScope = hlp.ItemToLocalSettings(changedData.Form_Network_AntiCryptorScope);
    AntiCryptorScope.SpecificFolders = changedData.Form_Network_AntiCryptorScope.UiFolders.map(function(node) { return node.Name })
    delete AntiCryptorScope.UiFolders;
    Object.assign(TASK_ADDITIONAL_PARAMS.Settings, lo.pick(AntiCryptorScope, formScopeValues));

    const AntiCryptorExcludes = hlp.ItemToLocalSettings(changedData.Form_Network_AntiCryptorExcludes);
    AntiCryptorExcludes.ExcludeFolders = changedData.Form_Network_AntiCryptorExcludes.UiFolders.map(function(node) { return node.Name })
    delete AntiCryptorExcludes.UiFolders
    Object.assign(TASK_ADDITIONAL_PARAMS.Settings, lo.pick(AntiCryptorExcludes, formExcludesValues))
}

function RbacToPolicyCache(activePolicyCache){
    const allFields = formCommonValues.concat(formScopeValues).concat(formExcludesValues);
    activePolicyCache.AntiCryptorPolicy = hlpPolicy.SetPolicyFieldsRbacLocked(activePolicyCache.AntiCryptorPolicy, allFields);
    activePolicyCache.PredefinedTasksPolicy = hlpPolicy.SetPolicyFieldsRbacLocked(activePolicyCache.PredefinedTasksPolicy, ['AntiCryptorPolicy']);
    return activePolicyCache;
}

module.exports = {
    defPolicySections,
    toView,
    toRaw,
    RbacToPolicyCache
};
