const lo     = require('lodash');
const utilsNasac = require('../../utils/nasac.js');
const hlpPolicy = require('../../../helpers_policy');
const hlp    = require('../../../helpers');
const hlpSch = require('./TaskScheduleHelper.js');

const defPolicySections = ['NetAppAntiCryptor','PredefinedTasksPolicy']
const formCommonValues = ['EnforcementMode','UseHeuristicLevel','HeuristicLevel']
const formAuthValues = ['cDOTIpAddress','VserverName','FpolicyName','FPolicyServerPort','cDOTNodesList','cDOTAccess','PriveledgedAccess']
const formExclValues = ['UseExcludes','ExcludeRules']

function toView(TASK_ADDITIONAL_PARAMS, activePolicyCache, formType) {
    if(formType === 'Schedule' ) {
        return hlpSch.toView(TASK_ADDITIONAL_PARAMS.Schedule, activePolicyCache.PredefinedTasksPolicy.NetAppAntiCryptorStartOptions);
    }

    if(formType!=='Main')
        return {}

    let data = {
        Form_Nas_AntiCryptCommon: hlp.localSettingsToView(TASK_ADDITIONAL_PARAMS.Settings,activePolicyCache.NetAppAntiCryptor,formCommonValues),
        Form_Nas_AntiCryptAuth: hlp.localSettingsToView(TASK_ADDITIONAL_PARAMS.Settings,activePolicyCache.NetAppAntiCryptor,formAuthValues),
        Form_Nas_AntiCryptExcludes : hlp.localSettingsToView(TASK_ADDITIONAL_PARAMS.Settings,activePolicyCache.NetAppAntiCryptor,formExclValues)
    }
    data.Form_Nas_AntiCryptAuth =  utilsNasac.toViewAuth(data.Form_Nas_AntiCryptAuth);
    data.Form_Nas_AntiCryptExcludes = utilsNasac.toViewExclusions(data.Form_Nas_AntiCryptExcludes);
    data.Form_Nas_AntiCryptCommon.LocalTask = true;
    data.Form_Nas_AntiCryptAuth.LocalTask = true;
    data.Form_Nas_AntiCryptExcludes.LocalTask = true;
    return data
}

async function toRawEx(kscApi, TASK_ADDITIONAL_PARAMS, changedData, formType, hostId) {
    if(formType === 'Schedule' ) {
        hlpSch.toRaw(changedData, TASK_ADDITIONAL_PARAMS.Schedule);
        return;
    }

    if(formType!=='Main')
        return;
    // common
    const AcCommon = hlp.ItemToLocalSettings(changedData.Form_Nas_AntiCryptCommon);
    Object.assign(TASK_ADDITIONAL_PARAMS.Settings, lo.pick(AcCommon, formCommonValues))
    // auth
    changedData.Form_Nas_AntiCryptAuth = await utilsNasac.toRawSettAuth(changedData.Form_Nas_AntiCryptAuth, kscApi, hostId);
    const AcAuth = hlp.ItemToLocalSettings(changedData.Form_Nas_AntiCryptAuth);
    Object.assign(TASK_ADDITIONAL_PARAMS.Settings, lo.pick(AcAuth, formAuthValues))
    // excludes
    const AcExcl = utilsNasac.toRowSettExclusions(changedData.Form_Nas_AntiCryptExcludes);
    Object.assign(TASK_ADDITIONAL_PARAMS.Settings, lo.pick(AcExcl, formExclValues));
}

function RbacToPolicyCache(activePolicyCache){
    const allFields = formCommonValues.concat(formAuthValues).concat(formExclValues);
    activePolicyCache.NetAppAntiCryptor = hlpPolicy.SetPolicyFieldsRbacLocked(activePolicyCache.NetAppAntiCryptor, allFields);
    activePolicyCache.PredefinedTasksPolicy = hlpPolicy.SetPolicyFieldsRbacLocked(activePolicyCache.PredefinedTasksPolicy, ['NetAppAntiCryptorStartOptions']);
    return activePolicyCache;
}

module.exports = {
    defPolicySections,
    toView,
    toRawEx,
    RbacToPolicyCache
};
