const { plugin: { pluginScope } } = require('@kl/constants')
const lo = require('lodash')
const hlp = require('../../helpers')
const hlpPolicy = require('../../helpers_policy')
const consts = require('../../consts')
const policyConverters = require('./sections/index.js')

function Policy () {
    let changesList = []
    let policiesCache = {}
    let curPolicyId = -1

    function AddToChangesList (arrSections) {
        arrSections = Array.isArray(arrSections) ? arrSections : [arrSections]
        for (let policySection of arrSections) {
            if (policySection != '' && !changesList.includes(policySection)) {
                changesList.push(policySection)
            }
        }
    }

    function UpdatePoliciesCacheAsDefault (arrSections) {
        arrSections = Array.isArray(arrSections) ? arrSections : [arrSections]
        for (let policySection of arrSections) {
            if (policySection != '' && policiesCache[policySection] === undefined ) {
                policiesCache[policySection] = hlpPolicy.DefaultPolicyObject(policySection)
            }
        }
    }

    async function UpdateRbac (api, id, rowData, sectionId) {
        let policyData = hlp.serialize(rowData);
        try {
            const readOnlyNodes = await api.HstAccessControl.GetPolicyReadonlyNodes({
                IGroupId: id,
                szwProduct: consts.wstrProduct,
                szwVersion: consts.wstrVersion,
                szwSectionName: sectionId,
                pPolicySection: policyData.Policy!==undefined ? policyData.Policy.value : policyData,
            });

            if (readOnlyNodes.result!==undefined ) {
                let changingData = rowData.Policy!==undefined ? rowData.Policy.KLPRSS_Val : rowData;
                hlpPolicy.SetPolicyFieldsRbacLocked(changingData, readOnlyNodes.result[sectionId]);
            }
        }
        catch (error) {
            console.error(error);
        }

        return rowData;
    }
    /*
     * Метод добавляет в cache политик новые секции
     */
    async function UpdatePoliciesCache (api, id, arrSections, parentId) {
        arrSections = Array.isArray(arrSections) ? arrSections : [arrSections]

        if (id === -1) {
            UpdatePoliciesCacheAsDefault(arrSections)
            return
        }

        let { result: identifier } = await api.Policy.GetPolicyContents({ nPolicy: id })

        for (let i = 0; i < arrSections.length; i++) {
            const policySection = arrSections[i]
            if( policiesCache[policySection] !== undefined ){
                continue;
            }

            const SsPolicySection = consts.policySsSections[policySection]
            if(SsPolicySection === undefined)
                continue

            let sectionData = await api.SsContents.Ss_Read({
                wstrID: identifier,
                wstrProduct: consts.wstrProduct,
                wstrVersion: consts.wstrVersion,
                wstrSection: SsPolicySection,
                skipDeserialization: true})

            let error = lo.get(sectionData, 'PxgError')
            if (error) {
                policiesCache[policySection] = hlpPolicy.DefaultPolicyObject(policySection)
            }
            else {
                let rowData = (sectionData=== undefined) ? hlpPolicy.DefaultPolicyObject(policySection) : hlp.deserialize(lo.get(sectionData, 'result')).KLPRSS_Val;
                rowData = await UpdateRbac (api, parentId, rowData, SsPolicySection);
                policiesCache[policySection] = rowData;
            }
        }
        api.SsContents.Ss_Release({ wstrID: identifier })
    }

    async function SaveDefaultPolicySections (api, id) {
        if( id === -1 )
            return

        let { result: identifier } = await api.Policy.GetPolicyContents({ nPolicy: id })
        for (let keySection in consts.policySsSections) {
            if( changesList[keySection] !== undefined )
                continue;

            const SsPolicySection = consts.policySsSections[keySection]
            let DataToSave = { KLPRSS_Val: hlpPolicy.DefaultPolicyObject(keySection) }
            if( DataToSave.KLPRSS_Val.length === 0 )
                continue;

            let result = hlp.serialize(DataToSave)
            result.KLPRSS_Mnd = false

            await api.SsContents.Ss_Replace({
                wstrID: identifier,
                wstrProduct: consts.wstrProduct,
                wstrVersion: consts.wstrVersion,
                wstrSection: SsPolicySection,
                pNewData: result,
                skipSerialization: true
            })
            await api.SsContents.Ss_Apply({ wstrID: identifier })
        }
        api.SsContents.Ss_Release({ wstrID: identifier })
    }
    /*
     * Called via api.js in getFormModel
     */
    async function getPolicyData (api, id, parentId, section) {
        if(curPolicyId != id) {
            curPolicyId = id
            policiesCache = {}
            changesList = []
        }

        if (section === undefined || policyConverters[section] === undefined)
            return {}

        if (policyConverters[section].sections !== undefined) {
            await UpdatePoliciesCache(api, id, policyConverters[section].sections, parentId);
        }

        if (policyConverters[section].toViewEx !== undefined) {
            return await policyConverters[section].toViewEx(api, policiesCache)
        }

        if (policyConverters[section].toView !== undefined) {
            return policyConverters[section].toView(policiesCache)
        }

        return {}
    }
    /*
     * Called via api.js in setFormModel
     */
    async function updatePolicyData (kscApi, section, data) {
        if (section === undefined || policyConverters[section] === undefined)
            return
        if (policyConverters[section].sections !== undefined) {
            AddToChangesList(policyConverters[section].sections)
        }

        if(policyConverters[section].toRawEx!==undefined)
            await policyConverters[section].toRawEx(kscApi, data, policiesCache)
        else
            policyConverters[section].toRaw(data, policiesCache)
    }
    /*
     * Called via api.js in beforeApplyChanges
     */
    function commitPolicyData (api, scope) {
    }
    /*
     * Called via api.js in applyChanges
     */
    async function savePolicyData (api, id, scope) {
        if (scope === pluginScope.newObject) {
            SaveDefaultPolicySections (api, id)
        }

        if(changesList.length === 0 )
            return

        let { result: identifier } = await api.Policy.GetPolicyContents({ nPolicy: id })
        for (let policySection of changesList) {
            const SsPolicySection = consts.policySsSections[policySection]
            let DataToSave = { KLPRSS_Val: policiesCache[policySection] }
            let result = hlp.serialize(DataToSave)
            result.KLPRSS_Mnd = false

            await api.SsContents.Ss_Replace({
                wstrID: identifier,
                wstrProduct: consts.wstrProduct,
                wstrVersion: consts.wstrVersion,
                wstrSection: SsPolicySection,
                pNewData: result,
                skipSerialization: true
            })
            await api.SsContents.Ss_Apply({ wstrID: identifier })
        }

        api.SsContents.Ss_Release({ wstrID: identifier })

        changesList = []
        policiesCache = {}
    }

    return {
        getPolicyData,
        updatePolicyData,
        commitPolicyData,
        savePolicyData
    }
}

module.exports = Policy
