const { helpers } = require('@kl/openapi-client')
const lo = require('lodash')
const hlpPolicy = require('../../helpers_policy')

const TaskTypeSettingsValue = 'PluginTaskType'
const taskConverters = {
    AutoRuleGenerator: require('./types/AutoRuleGenerator.js'),
    CheckAppIntegrity: require('./types/CheckAppIntegrity.js'),
    CheckFilesIntegrity: require('./types/CheckFilesIntegrity.js'),
    InstallKey: require('./types/InstallKey.js'),
    DeviceRuleGenerator:require('./types/DeviceRuleGenerator.js'),
    OnDemand:require('./types/OnDemand.js'),
	RetranslateUpdates: require('./types/RetranslateUpdates.js'),
    UpdateBases: require('./types/UpdateBases.js'),
	UpdateComponents: require('./types/UpdateComponents.js'),
}

function taskInfo() {
    let taskAdditionalParams = { } //initial data
    let currentTaskType = ''
    let curTaskId = ''

    async function getTaskDataCache (kscApi, taskId) {
        if( curTaskId === taskId || taskId === '')
            return
        let result = await hlpPolicy.GetTaskData(kscApi, taskId);
        if ( result && result.TASK_ADDITIONAL_PARAMS !==undefined ) {
            taskAdditionalParams = lo.clone(result.TASK_ADDITIONAL_PARAMS);
            currentTaskType = result.TASK_NAME;
            curTaskId = taskId.toString();
        }
        else{
            curTaskId = ''
            currentTaskType = ''
            taskAdditionalParams = {}
        }
    }

    function mapTaskData(formType) {
        let computedData = {}
        let converter = taskConverters[currentTaskType]
        if (converter) {
            computedData = converter.toView(taskAdditionalParams, formType)
        }
        computedData[TaskTypeSettingsValue] = currentTaskType
        return computedData
    }

    async function getTaskData ({kscApi, taskId, hostId, formType}) {
        console.log('kess_windows:> getTaskData ' + JSON.stringify(arguments))
        await getTaskDataCache (kscApi, taskId);
        return mapTaskData(formType)
    }

    function getTaskAdditionalParams() {
        console.log('kess_windows:GroupTask:getTaskAdditionalParams ' + JSON.stringify(taskAdditionalParams));
        return taskAdditionalParams;
    }

    async function updateTaskData ({kscApi, modelData, formType}) {
        let converter = taskConverters[currentTaskType]
        if (converter) {
            converter.toRaw(taskAdditionalParams, modelData, formType)
        }
    }

    async function getWizardTaskData({kscApi, taskType, formType, defData}) {
        let tmpData = {PxgRetVal:lo.clone(defData)};
        let Data = helpers.serializer.deserializeKscRequest({body: tmpData}).result
        if (!Data || !Data.TASK_ADDITIONAL_PARAMS) {
            return {}
        }

        taskAdditionalParams = lo.clone(Data.TASK_ADDITIONAL_PARAMS)
        currentTaskType = taskType
        return mapTaskData(formType)
    }

    return {
        getTaskData,
        getTaskAdditionalParams,
        updateTaskData,
        getWizardTaskData
    }
}

module.exports = taskInfo
