const lo = require('lodash')
const fs = require('fs')
const path = require('path')

const buffer = require('buffer');
const { TextDecoder, TextEncoder } =  require('util')

const ConfigFileName = 'config.ini'

const DEF_INSTALLDIR = 'INSTALLDIR="%ProgramFiles%\Kaspersky Lab\Kaspersky Embedded Systems Security\"';
const DEF_INSTALL_PARAMS = 'NOAVBASES=0 PRESCAN=0 RUNRTP=1 ADDMSEXCLUSION=1 ADDKLEXCLUSION=1';
const DEF_INSTALL_FEATURES = 'ADDLOCAL=Ods,Oas,AKIntegration,PerfMonCounters,SnmpSupport,Shell,TrayApp,Ksn,AppCtrl,DevCtrl,AVProtection,RamDisk,Fim,AntiExploit,LogInspector,IDS'
const SureInstallFeatures = 'AVProtection,RamDisk,AKIntegration,Shell';
const ManagedFeatures = [
    'Oas','Ods','Ksn','AppCtrl','DevCtrl','Firewall','Fim', 'IDS','AntiExploit','LogInspector','PerfMonCounters','SnmpSupport','Shell','TrayApp'
];

function AddFeature(featuresList, feature) {
    if (featuresList.length > 0)
        featuresList += ',';
    featuresList += feature;
    return featuresList;
}

function packageInfo() {
    let currentPackageId = -1

    function GetBoolKeyValue(keys, key) {
        return keys[key]!==undefined && keys[key]==='1' ? true : false
    }

    function GetBoolCommandLineProp(data, key, pref) {
        if(data[key]===undefined)
            return ''
        const value = data[key]===true ? '1' : '0'
        return (pref + key + '=' + value)
    }

    function GetKeys(rowStr) {
        let str = rowStr.replace('= ', '=')
        str = str.replace(' =', '=')
        let keys = {}
        while (true) {
            let indexA = str.indexOf(' ')
            if(indexA=== -1 && str.length > 0 )
                indexA = str.length;

            let token = str.substring(0, indexA)
            let indexB = token.indexOf('=')
            if(indexB!==-1 && token.length){
                if(token[indexB+1]==='"') {
                    indexA = str.indexOf('"', indexA + 2)
                    token = str.substring(0, indexA)
                    keys[token.substring(0, indexB)] = token.substring(indexB+2)
                }
                else
                    keys[token.substring(0, indexB)] = token.substring(indexB+1)
            }

            if(str.length===0 || indexA > str.length - 1)
                break;
            str = str.substring(indexA + 1)
        }
        return keys;
    }

    function toView (keys, import_settings_path) {
        const bNoAvBases = GetBoolKeyValue(keys, 'NOAVBASES');
        let data = {
            features: {  AKIntegration: true },
            NOAVBASES: bNoAvBases,
            PRESCAN: GetBoolKeyValue(keys, 'PRESCAN'),
            RUNRTP: bNoAvBases ? false : GetBoolKeyValue(keys, 'RUNRTP'),
            ADDMSEXCLUSION: GetBoolKeyValue(keys, 'ADDMSEXCLUSION'),
            ADDKLEXCLUSION: GetBoolKeyValue(keys, 'ADDKLEXCLUSION'),
            SKIPCVEWINDOWS10: GetBoolKeyValue(keys, 'SKIPCVEWINDOWS10'),
            INSTALLDIR: keys.INSTALLDIR!==undefined ? keys.INSTALLDIR : '',
            ImportPath: (import_settings_path!==undefined && import_settings_path!=='undefined') ? import_settings_path : ''
        };

        let installed = keys['ADDLOCAL'].split(',');
        if(keys.ADD_ADV!==undefined && keys.ADD_ADV!==''){
            installed = installed.concat(keys.ADD_ADV.split(','));
        }
        for(let item of installed){
            data.features[item] = installed.includes(item)
        }

        return data;
    }

    function toRow (changedData) {
        let decoded = GetBoolCommandLineProp(changedData, 'NOAVBASES', '');
        decoded += GetBoolCommandLineProp(changedData, 'PRESCAN', ' ');
        decoded += GetBoolCommandLineProp(changedData, 'RUNRTP', ' ');
        decoded += GetBoolCommandLineProp(changedData, 'ADDMSEXCLUSION', ' ');
        decoded += GetBoolCommandLineProp(changedData, 'ADDKLEXCLUSION', ' ');
        decoded += GetBoolCommandLineProp(changedData, 'SKIPCVEWINDOWS10', ' ');
        decoded += ' INSTALLDIR="' + changedData.INSTALLDIR + '"';

        let AddLocal = '';
        let Remove = '';
        for(let item of ManagedFeatures)  {
            if(changedData.NOAVBASES===true && (item==='Oas' || item==='Ods') ){
                Remove = AddFeature(Remove, item);
                continue;
            }

            if(changedData.features[item]!==undefined && changedData.features[item]===true){
                AddLocal = AddFeature(AddLocal, item);
            }
            else{
                Remove = AddFeature(Remove, item);
            }
        }

        decoded += ' ADDLOCAL=' + SureInstallFeatures + ',' + AddLocal;
        if(Remove.length > 0)
            decoded += ' REMOVE=' + Remove;

        // update advanced features
        AddLocal = '';
        Remove = '';
        if(changedData.features.IDS!==undefined && changedData.features.IDS===true && changedData.NOAVBASES===false){
            AddLocal = AddFeature(AddLocal, 'IDS');
        }
        else{
            Remove = AddFeature(Remove, 'IDS');
        }
        if(AddLocal.length > 0)
            decoded += ' ADD_ADV=' + AddLocal;
        if(Remove.length > 0)
            decoded += ' REMOVE_ADV=' + Remove;

        decoded += ('\n' + changedData.ImportPath);
        return decoded;
    }

    async function getPackageData ({kscApi, packageId, form}) {
        currentPackageId = parseInt(packageId)
        let iniInfo = await kscApi.PackagesApi.ReadPkgCfgFile({nPackageId: currentPackageId, wstrFileName:ConfigFileName})
        let buff = Buffer.from(iniInfo.result, 'base64')
        let decoded = new TextDecoder('utf-16').decode(buff)
        const [command_line, import_settings_path] = decoded.split('\n')
        let keys = GetKeys(command_line)
        return toView (keys, import_settings_path)
    }

    async function savePackageData ({kscApi, modelData, form}) {
        if( form!=='InstallPackage' )
            return
        const decoded = toRow(modelData);
        let bytes = Buffer.from(decoded, 'utf-8')
        let bufStr16 = buffer.transcode(bytes, 'utf8', 'utf16le');
        let base64Str = bufStr16.toString('base64');
        await kscApi.PackagesApi.WritePkgCfgFile({nPackageId: currentPackageId, wstrFileName: ConfigFileName, pData: base64Str})
    }

    return {
        getPackageData,
        savePackageData
    }
}

module.exports = packageInfo